==> folder-of-fortune-0.0.3.tar.gz: folder-of-fortune-0.0.3/src/folder-of-fortune.c <==
/* Copyright (C) 2005 Free Software Foundation, Inc.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Kang Jeong-Hee <Keizi@mail.co.kr>
 *
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string.h>

#include <glib.h>
#include <glib/gi18n-lib.h>
#include <libgnomevfs/gnome-vfs.h>
#include <gconf/gconf-client.h>
#include <gconf/gconf-value.h>
#include <libnautilus-extension/nautilus-extension-types.h>
#include <libnautilus-extension/nautilus-menu-provider.h>

/* libnautilus-private/nautilus-file-utilities.h
 * Nautilus will provide runtime symbol. */
char *	nautilus_get_desktop_directory_uri	(void);

#define FOLDER_OF_FORTUNE_TYPE (folder_of_fortune_get_type ())
#define FOLDER_OF_FORTUNE(o)   (G_TYPE_CHECK_INSTANCE_CAST ((o), FOLDER_OF_FORTUNE_TYPE))

typedef struct {
	GObject parent;
} FolderOfFortune;

typedef struct {
	GObjectClass parent_class;
} FolderOfFortuneClass;

static GType		 this_type = 0;
static GObjectClass	*parent_class = NULL;
static GConfClient	*default_client = NULL;

static GMarkupParseContext *ctxt;
static GMarkupParser parser = {
	NULL, NULL, NULL, NULL, NULL
	/*
	start_element_handler,
	end_element_handler,
	text_handler,
	passthrought_handler,
	error_handler
	*/
};

#define	SEED_FILE_KEY	"/apps/folder_of_fortune/seed_file"

/* FIXME: three key-value pair lists for prefix, seed, suffix */
static gpointer seed_pool = {
	NULL,
};

static GType
folder_of_fortune_get_type (void)
{
	return this_type;
}

/* TODO: pull from seed_pool */
static gchar *
get_fortunate_name (void)
{
	return g_strdup_printf ("FIXME_%d", g_random_int_range (0, 1000));
}

static void
create_folder (NautilusFileInfo *current_folder)
{
	gchar *parent, *new;
	GnomeVFSURI *parent_uri, *uri = NULL;
	GnomeVFSResult res;
	guint loop = 0;

	parent = nautilus_file_info_get_uri (current_folder);
	if (eel_uri_is_desktop (parent)) {
		g_free (parent);
		parent = nautilus_get_desktop_directory_uri ();
	}
	parent_uri = gnome_vfs_uri_new (parent);

	/* FIXME: infinite loop when every possible names created */
	do {
		if (uri) gnome_vfs_uri_unref (uri);
		new = get_fortunate_name ();
		uri = gnome_vfs_uri_append_file_name (parent_uri, new);
		g_free (new);
	} while (gnome_vfs_uri_exists (uri));

	res = gnome_vfs_make_directory_for_uri (uri, GNOME_VFS_PERM_USER_ALL |
			GNOME_VFS_PERM_GROUP_READ | GNOME_VFS_PERM_GROUP_EXEC |
			GNOME_VFS_PERM_OTHER_READ | GNOME_VFS_PERM_OTHER_EXEC);
	if (res == GNOME_VFS_OK) {
		new = gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_NONE);
		gnome_url_show (new, NULL);
		g_free (new);
	} else {
		g_debug ("Error creating folder : %s",
			 gnome_vfs_uri_get_path (uri));
	}
	
	gnome_vfs_uri_unref (uri);
	gnome_vfs_uri_unref (parent_uri);
}

static void
context_menu_activate (NautilusMenuItem *item,
		       NautilusFileInfo *current_folder)
{
	create_folder (current_folder);
}

static gboolean
is_writable (NautilusFileInfo *cur)
{
	gchar *uri;
	GnomeVFSFileInfo *info;
	gboolean writable;

	uri = nautilus_file_info_get_uri (cur);
	if (eel_uri_is_in_trash (uri)) {
		g_free (uri);
		return FALSE;
	}
	if (eel_uri_is_desktop (uri)) {
		g_free (uri);
		return TRUE;
	}

	info = nautilus_file_info_get_vfs_file_info (cur);
	writable = info->permissions & (GNOME_VFS_PERM_USER_WRITE |
					GNOME_VFS_PERM_GROUP_WRITE |
					GNOME_VFS_PERM_OTHER_WRITE);
	gnome_vfs_file_info_unref (info);

	return writable;
}

static GList *
folder_of_fortune_get_background_items (NautilusMenuProvider *provider,
					GtkWidget *window,
					NautilusFileInfo *current_folder)
{
	GList *items = NULL;
	NautilusMenuItem *item;

	g_return_val_if_fail (is_writable (current_folder), items);

	item = nautilus_menu_item_new ("Folder of fortune::create_any_folder",
				       _("Create any folder"),
				       _("Create a new folder of random name"),
				       NULL);
	g_signal_connect (item, "activate",
			  G_CALLBACK (context_menu_activate), current_folder);
	items = g_list_prepend (items, item);

	return items;
}

static void
folder_of_fortune_class_init (FolderOfFortuneClass *class)
{
	parent_class = g_type_class_peek_parent (class);
}
static void
folder_of_fortune_init (FolderOfFortune *self)
{
}
static void
menu_provider_iface_init (NautilusMenuProviderIface *iface)
{
	iface->get_background_items = folder_of_fortune_get_background_items;
}

void
folder_of_fortune_register_type (GTypeModule *module)
{
	static const GTypeInfo info = {
		sizeof (FolderOfFortuneClass),
		(GBaseInitFunc) NULL,
		(GBaseFinalizeFunc) NULL,
		(GClassInitFunc) folder_of_fortune_class_init,
		NULL,
		NULL,
		sizeof (FolderOfFortune),
		0,
		(GInstanceInitFunc) folder_of_fortune_init
		};
		static const GInterfaceInfo menu_provider_iface_info = {
		(GInterfaceInitFunc)menu_provider_iface_init,
		NULL,
		NULL
	};

	this_type = g_type_module_register_type (module,
						 G_TYPE_OBJECT,
						 "FolderOfFortune",
						 &info, 0);
	g_type_module_add_interface (module,
				     this_type,
				     NAUTILUS_TYPE_MENU_PROVIDER,
				     &menu_provider_iface_info);
}

/* TODO: free the pool */
static void
seed_pool_free (gpointer data)
{
	;
}

static void
seed_populate (const gchar *seed_file)
{
	gchar *contents;
	gsize length;

	if (!g_file_get_contents (seed_file, &contents, &length, NULL)) {
		contents = _("<prefix>absolute</prefix><prefix>totally</prefix><seed>nothing</seed><seed>empty</seed><suffix>default</suffix>");
		length = strlen (contents);
	}
	ctxt = g_markup_parse_context_new (&parser, 0,
					   seed_pool,
					   (GDestroyNotify) seed_pool_free);
	g_markup_parse_context_parse (ctxt, contents, length, NULL);
}

static void
seed_file_changed (GConfClient *client,
		   guint cnxn_id,
		   GConfEntry *entry,
		   gpointer user_data)
{
	seed_populate (gconf_value_get_string (gconf_entry_get_value (entry)));
}

void
nautilus_module_initialize (GTypeModule *module)
{
	gchar *seed_file;

	folder_of_fortune_register_type (module);

	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");

	default_client = gconf_client_get_default ();
	gconf_client_notify_add (default_client, SEED_FILE_KEY,
				 (GConfClientNotifyFunc) seed_file_changed,
				 NULL, NULL, NULL);
	seed_file = gconf_client_get_string (default_client,
					     SEED_FILE_KEY, NULL);
	seed_populate (seed_file);
	g_free (seed_file);
}

void
nautilus_module_shutdown (void)
{
	if (default_client)
		g_object_unref (default_client);
	default_client = NULL;

	g_markup_parse_context_free (ctxt);
}

void
nautilus_module_list_types (const GType **types,
			    int *num_types)
{
	static GType type_list[1];

	type_list[0] = FOLDER_OF_FORTUNE_TYPE;
	*types = type_list;
	*num_types = G_N_ELEMENTS (type_list);
}

==> folder-of-fortune-0.0.4.tar.gz: folder-of-fortune-0.0.4/src/folder-of-fortune.c <==
/* Copyright (C) 2005 Free Software Foundation, Inc.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Kang Jeong-Hee <Keizi@mail.co.kr>
 *
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string.h>

#include <glib.h>
#include <glib/gi18n-lib.h>
#include <libgnomevfs/gnome-vfs.h>
#include <gconf/gconf-client.h>
#include <gconf/gconf-value.h>
#include <libnautilus-extension/nautilus-extension-types.h>
#include <libnautilus-extension/nautilus-menu-provider.h>

/* libnautilus-private/nautilus-file-utilities.h
 * Nautilus will provide runtime symbol. */
char *	nautilus_get_desktop_directory_uri	(void);

#define FOLDER_OF_FORTUNE_TYPE (folder_of_fortune_get_type ())
#define FOLDER_OF_FORTUNE(o)   (G_TYPE_CHECK_INSTANCE_CAST ((o), FOLDER_OF_FORTUNE_TYPE))

typedef struct {
	GObject parent;
} FolderOfFortune;

typedef struct {
	GObjectClass parent_class;
} FolderOfFortuneClass;

static GType		 this_type = 0;
static GObjectClass	*parent_class = NULL;
static GConfClient	*default_client = NULL;

#define	SEED_FILE_KEY	"/apps/folder-of-fortune/seed_file"
#define	OPEN_FOLDER_KEY	"/apps/folder-of-fortune/open_folder"

static gboolean open_folder = FALSE;

typedef struct {
	GList *cur;

	GList *seed;
	GList *prefix;
	GList *suffix;
} SeedPool;
static SeedPool seed_pool;

static void
start_element_handler (GMarkupParseContext *context,
		       const gchar         *element_name,
		       const gchar        **attribute_names,
		       const gchar        **attribute_values,
		       gpointer             user_data,
		       GError             **error)
{
	SeedPool *p = (SeedPool *) user_data;

	if (!strcmp (element_name, "seed")) {
		p->cur = p->seed;
	} else
	if (!strcmp (element_name, "prefix")) {
		p->cur = p->prefix;
	} else
	if (!strcmp (element_name, "suffix")) {
		p->cur = p->suffix;
	} else {
		p->cur = NULL;
	}
}

static void
end_element_handler (GMarkupParseContext *context,
		     const gchar         *element_name,
		     gpointer             user_data,
		     GError             **error)
{
	SeedPool *p = (SeedPool *) user_data;

	p->cur = NULL;
}

static void
text_handler (GMarkupParseContext *context,
	      const gchar         *text,
	      gsize                text_len,
	      gpointer             user_data,
	      GError             **error)
{
	SeedPool *p = (SeedPool *) user_data;

	if (p->cur == NULL) return;

	p->cur = g_list_append (p->cur, (gpointer) g_strdup (text));
}

static GMarkupParseContext *ctxt;
static GMarkupParser parser = {
	start_element_handler,
	end_element_handler,
	text_handler,
	NULL,
	NULL
};

static GType
folder_of_fortune_get_type (void)
{
	return this_type;
}

static gchar *
get_fortunate_name_internal (GList *pool)
{
	return g_list_nth_data (pool,
				g_random_int_range (1, g_list_length (pool)));
}

static gchar *
get_fortunate_name (const guint repeat)
{
	gchar *data, *ret, *tmp = NULL;

	ret = g_strdup (get_fortunate_name_internal (seed_pool.seed));

	if (repeat > 1) {
		data = get_fortunate_name_internal (seed_pool.prefix);
		tmp = ret;
		ret = g_strdup_printf ("%s %s", data, ret);
		g_free (tmp);
	}
	if (repeat > 2) {
		data = get_fortunate_name_internal (seed_pool.suffix);
		tmp = ret;
		ret = g_strdup_printf ("%s %s", ret, data);
		g_free (tmp);
	}

	return ret;
}

static void
create_folder (NautilusFileInfo *current_folder)
{
	gchar *parent, *new, *new_local = NULL;
	GnomeVFSURI *parent_uri, *uri = NULL;
	GnomeVFSResult res;
	guint repeat = 0;
	gboolean is_local = FALSE;

	parent = nautilus_file_info_get_uri (current_folder);
	if (eel_uri_is_desktop (parent)) {
		g_free (parent);
		parent = nautilus_get_desktop_directory_uri ();
	}
	parent_uri = gnome_vfs_uri_new (parent);
	is_local = gnome_vfs_uri_is_local (parent_uri);

	do {
		if (uri) gnome_vfs_uri_unref (uri);
		repeat += 1;
		new = get_fortunate_name (repeat);
		if (is_local) {
			new_local = g_filename_from_utf8 (new, -1,
							  NULL, NULL, NULL);
			uri = gnome_vfs_uri_append_file_name (parent_uri,
							      new_local);
			g_free (new_local);
		} else {
			uri = gnome_vfs_uri_append_file_name (parent_uri, new);
			g_free (new);
		}
	} while (gnome_vfs_uri_exists (uri));
	repeat = 0;

	res = gnome_vfs_make_directory_for_uri (uri, GNOME_VFS_PERM_USER_ALL |
			GNOME_VFS_PERM_GROUP_READ | GNOME_VFS_PERM_GROUP_EXEC |
			GNOME_VFS_PERM_OTHER_READ | GNOME_VFS_PERM_OTHER_EXEC);
	if (res == GNOME_VFS_OK) {
		if (open_folder) {
			new = gnome_vfs_uri_to_string (uri,
						       GNOME_VFS_URI_HIDE_NONE);
			gnome_url_show (new, NULL);
			g_free (new);
		}
	} else {
		g_debug ("Error creating folder : %s",
			 gnome_vfs_uri_get_path (uri));
	}
	
	gnome_vfs_uri_unref (uri);
	gnome_vfs_uri_unref (parent_uri);
}

static void
context_menu_activate (NautilusMenuItem *item,
		       NautilusFileInfo *current_folder)
{
	create_folder (current_folder);
}

static gboolean
is_writable (NautilusFileInfo *cur)
{
	gchar *uri;
	GnomeVFSURI *vfs_uri;
	GnomeVFSFileInfo *info;
	gboolean writable = FALSE;

	uri = nautilus_file_info_get_uri (cur);
	if (eel_uri_is_desktop (uri)) {
		writable = TRUE;
		goto early_done;
	}
	if (eel_uri_is_in_trash (uri)) {
		writable = FALSE;
		goto early_done;
	}

	vfs_uri = gnome_vfs_uri_new (uri);
	info = gnome_vfs_file_info_new ();
	if (gnome_vfs_get_file_info_uri (vfs_uri, info,
			GNOME_VFS_FILE_INFO_GET_ACCESS_RIGHTS) != GNOME_VFS_OK)
	{
		writable = FALSE;
		goto done;
	}

	if (gnome_vfs_uri_is_local (vfs_uri)) {
		writable = (info->permissions & GNOME_VFS_PERM_ACCESS_WRITABLE)
					     == GNOME_VFS_PERM_ACCESS_WRITABLE;
	} else {
		writable = info->permissions & (GNOME_VFS_PERM_USER_WRITE |
						GNOME_VFS_PERM_GROUP_WRITE |
						GNOME_VFS_PERM_OTHER_WRITE);
	}

done:
	gnome_vfs_uri_unref (vfs_uri);
	gnome_vfs_file_info_unref (info);
early_done:
	g_free (uri);

	return writable;
}

static GList *
folder_of_fortune_get_background_items (NautilusMenuProvider *provider,
					GtkWidget *window,
					NautilusFileInfo *current_folder)
{
	GList *items = NULL;
	NautilusMenuItem *item;

	g_return_val_if_fail (is_writable (current_folder), items);

	item = nautilus_menu_item_new ("Folder of fortune::create_any_folder",
				       _("Create any folder"),
				       _("Create a new folder of random name"),
				       NULL);
	g_signal_connect (item, "activate",
			  G_CALLBACK (context_menu_activate), current_folder);
	items = g_list_prepend (items, item);

	return items;
}

static void
folder_of_fortune_class_init (FolderOfFortuneClass *class)
{
	parent_class = g_type_class_peek_parent (class);
}
static void
folder_of_fortune_init (FolderOfFortune *self)
{
}
static void
menu_provider_iface_init (NautilusMenuProviderIface *iface)
{
	iface->get_background_items = folder_of_fortune_get_background_items;
}

void
folder_of_fortune_register_type (GTypeModule *module)
{
	static const GTypeInfo info = {
		sizeof (FolderOfFortuneClass),
		(GBaseInitFunc) NULL,
		(GBaseFinalizeFunc) NULL,
		(GClassInitFunc) folder_of_fortune_class_init,
		NULL,
		NULL,
		sizeof (FolderOfFortune),
		0,
		(GInstanceInitFunc) folder_of_fortune_init
		};
		static const GInterfaceInfo menu_provider_iface_info = {
		(GInterfaceInitFunc)menu_provider_iface_init,
		NULL,
		NULL
	};

	this_type = g_type_module_register_type (module,
						 G_TYPE_OBJECT,
						 "FolderOfFortune",
						 &info, 0);
	g_type_module_add_interface (module,
				     this_type,
				     NAUTILUS_TYPE_MENU_PROVIDER,
				     &menu_provider_iface_info);
}

static void
seed_pool_free (gpointer data)
{
	SeedPool *p = (SeedPool *) data;

	g_list_foreach (p->seed, (GFunc) g_free, NULL);
	g_list_foreach (p->prefix, (GFunc) g_free, NULL);
	g_list_foreach (p->suffix, (GFunc) g_free, NULL);

	g_list_free (p->seed);
	g_list_free (p->prefix);
	g_list_free (p->suffix);

	p->cur = NULL;
}

static void
seed_populate (const gchar *seed_file)
{
	gchar *contents;
	gsize length;

	if (!g_file_get_contents (seed_file, &contents, &length, NULL)) {
		contents = _("<prefix>absolute</prefix><prefix>totally</prefix><seed>nothing</seed><seed>empty</seed><suffix>default</suffix>");
		length = strlen (contents);
	}
	ctxt = g_markup_parse_context_new (&parser, 0,
					   &seed_pool,
					   (GDestroyNotify) seed_pool_free);
	g_markup_parse_context_parse (ctxt, contents, length, NULL);
	g_markup_parse_context_end_parse (ctxt, NULL);

	if (contents) g_free (contents);
}

static void
seed_file_changed (GConfClient *client,
		   guint cnxn_id,
		   GConfEntry *entry,
		   gpointer user_data)
{
	seed_populate (gconf_value_get_string (gconf_entry_get_value (entry)));
}

static void
open_folder_changed (GConfClient *clieng,
		     guint cnxn_id,
		     GConfEntry *entry,
		     gpointer user_data)
{
	open_folder = gconf_value_get_bool (gconf_entry_get_value (entry));
}

void
nautilus_module_initialize (GTypeModule *module)
{
	gchar *seed_file;

	folder_of_fortune_register_type (module);

	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");

	seed_pool.cur = NULL;
	seed_pool.seed = g_list_alloc ();
	seed_pool.prefix = g_list_alloc ();
	seed_pool.suffix = g_list_alloc ();

	default_client = gconf_client_get_default ();

	seed_file = gconf_client_get_string (default_client,
					     SEED_FILE_KEY, NULL);
	gconf_client_notify_add (default_client, SEED_FILE_KEY,
				 (GConfClientNotifyFunc) seed_file_changed,
				 NULL, NULL, NULL);
	seed_populate (seed_file);
	g_free (seed_file);

	open_folder = gconf_client_get_bool (default_client,
					     OPEN_FOLDER_KEY, NULL);
	gconf_client_notify_add (default_client, OPEN_FOLDER_KEY,
				 (GConfClientNotifyFunc) open_folder_changed,
				 NULL, NULL, NULL);
}

void
nautilus_module_shutdown (void)
{
	if (default_client)
		g_object_unref (default_client);
	default_client = NULL;

	g_markup_parse_context_free (ctxt);
}

void
nautilus_module_list_types (const GType **types,
			    int *num_types)
{
	static GType type_list[1];

	type_list[0] = FOLDER_OF_FORTUNE_TYPE;
	*types = type_list;
	*num_types = G_N_ELEMENTS (type_list);
}
